@@ -2,7 +2,6 @@
 
 from __future__ import annotations
 
-import abc
 from typing import (
     TypeVar,
     Optional,
@@ -20,11 +19,11 @@ DeviceType = TypeVar("DeviceType", bound=Device)
 
 
 @attrs.define
-class DeviceConfiguration(abc.ABC, Generic[DeviceType]):
+class DeviceConfiguration(Generic[DeviceType]):
     """Contains static information about a device.
 
     This is an abstract class, generic in :data:`DeviceType` that stores the information
-    necessary to connect to a device and program it during a sequencer.
+    necessary to connect to a device and program it during a sequence.
 
     This information is meant to be encoded in a user-friendly way that might not be
     possible to be directly programmed on a device.
@@ -39,14 +38,9 @@ class DeviceConfiguration(abc.ABC, Generic[DeviceType]):
     Attributes:
         remote_server: Indicates the name of the computer on which the device should be
             instantiated.
-            If None, the device should be instantiated in the local process controlling
-            the experiment.
     """
 
-    remote_server: Optional[DeviceServerName] = attrs.field(
-        converter=attrs.converters.optional(str),
-        on_setattr=attrs.setters.convert,
-    )
+    remote_server: Optional[DeviceServerName] = attrs.field()
 
 
 DeviceConfigType = TypeVar("DeviceConfigType", bound=DeviceConfiguration)
