@@ -6,22 +6,15 @@ This module contains classes that can be used to construct complex tree structur
 represent user defined transformations.
 """
 
-import functools
-
-import cattrs.strategies
-
 from ._converter import converter, structure_evaluable_output
 from ._output_mapping import LinearInterpolation
-from .transformation import Transformation, evaluate, EvaluableOutput
-
-cattrs.strategies.include_subclasses(
+from ._transformation import (
     Transformation,
-    converter=converter,
-    union_strategy=functools.partial(
-        cattrs.strategies.configure_tagged_union, tag_name="type"
-    ),
+    evaluate,
+    EvaluableOutput,
+    evaluable_output_validator,
 )
-converter.register_structure_hook(EvaluableOutput, structure_evaluable_output)
+
 
 __all__ = [
     "Transformation",
@@ -29,4 +22,6 @@ __all__ = [
     "evaluate",
     "EvaluableOutput",
     "converter",
+    "evaluable_output_validator",
+    "structure_evaluable_output",
 ]
