@@ -2,13 +2,11 @@
 
 from . import channel_commands
 from . import compilation
-from . import instructions
 from . import trigger
 from ._controller import SequencerController
 from ._converter import converter
 from ._proxy import SequencerProxy
-from ._time_step import TimeStep
-from .compilation import SequencerCompiler
+from .compilation._compiler import SequencerCompiler
 from .configuration import (
     SequencerConfiguration,
     ChannelConfiguration,
@@ -16,6 +14,7 @@ from .configuration import (
     AnalogChannelConfiguration,
 )
 from .runtime import Sequencer
+from .timing import TimeStep
 
 __all__ = [
     "Sequencer",
@@ -26,9 +25,9 @@ __all__ = [
     "SequencerProxy",
     "SequencerController",
     "channel_commands",
-    "instructions",
     "converter",
     "TimeStep",
     "trigger",
     "compilation",
+    "SequencerCompiler",
 ]
