@@ -1,7 +1,6 @@
 """Define classes and functions to evaluate output for a sequencer."""
 
 from ._compiler import (
-    SequencerCompiler,
     InstructionCompilationParameters,
     compile_parallel_instructions,
 )
@@ -10,7 +9,6 @@ from ..channel_commands._channel_sources._trigger_compiler import (
 )
 
 __all__ = [
-    "SequencerCompiler",
     "TriggerableDeviceCompiler",
     "InstructionCompilationParameters",
     "compile_parallel_instructions",
