@@ -1,12 +1,10 @@
 """Allow to manage the high-level control of experiments."""
 
-from . import shot_timing
+from . import sequence_execution
 from .manager import ExperimentManager, Procedure
-from .sequence_runner import ShotRetryConfig
 
 __all__ = [
-    "ShotRetryConfig",
     "ExperimentManager",
     "Procedure",
-    "shot_timing",
+    "sequence_execution",
 ]
