@@ -1,6 +1,9 @@
 """Allows to interact with the storage of the experiment."""
 
+from . import copy
+from . import sql
 from ._device_configuration_collection import DeviceConfigurationCollection
+from ._exception_summary import TracebackSummary
 from ._experiment_session import ExperimentSession
 from ._path import PureSequencePath, InvalidPathFormatError
 from ._path_hierarchy import (
@@ -10,6 +13,7 @@ from ._path_hierarchy import (
     PathHasChildrenError,
 )
 from ._path_hierarchy import PathHierarchy
+from ._sequence import Sequence, Shot
 from ._sequence_collection import (
     SequenceCollection,
     PathIsSequenceError,
@@ -19,15 +23,15 @@ from ._sequence_collection import (
     InvalidStateTransitionError,
     SequenceNotEditableError,
     ShotNotFoundError,
+    SequenceNotCrashedError,
 )
-from ._session_maker import ExperimentSessionMaker
-from .async_session import AsyncExperimentSession
-from ._sequence import Sequence, Shot
+from ._session_maker import ExperimentSessionMaker, StorageManager
 from ._state import State
-from . import sql
+from .async_session import AsyncExperimentSession
 
 __all__ = [
     "ExperimentSession",
+    "StorageManager",
     "Sequence",
     "State",
     "Shot",
@@ -49,5 +53,8 @@ __all__ = [
     "PathNotFoundError",
     "PathIsRootError",
     "PathHasChildrenError",
+    "TracebackSummary",
+    "SequenceNotCrashedError",
     "sql",
+    "copy",
 ]
