@@ -4,8 +4,7 @@ from abc import abstractmethod
 from datetime import datetime
 from typing import Protocol, TYPE_CHECKING
 
-from returns.result import Result
-
+from caqtus.utils.result import Success, Failure
 from .._path import PureSequencePath
 from .._path_hierarchy import PathNotFoundError, PathIsRootError
 
@@ -21,11 +20,15 @@ class AsyncPathHierarchy(Protocol):
     @abstractmethod
     async def get_children(
         self, path: PureSequencePath
-    ) -> Result[set[PureSequencePath], PathNotFoundError | PathIsSequenceError]:
+    ) -> (
+        Success[set[PureSequencePath]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsSequenceError]
+    ):
         raise NotImplementedError
 
     @abstractmethod
     async def get_path_creation_date(
         self, path: PureSequencePath
-    ) -> Result[datetime, PathNotFoundError | PathIsRootError]:
+    ) -> Success[datetime] | Failure[PathNotFoundError] | Failure[PathIsRootError]:
         raise NotImplementedError
