@@ -15,4 +15,4 @@ def first(iterable: Iterable[T]) -> T:
     try:
         return next(iterator)
     except StopIteration:
-        raise ValueError("Iterable is empty")
+        raise ValueError("Iterable is empty") from None
