@@ -1,12 +1,11 @@
-from functools import partial
 from typing import Optional, Callable, Any, TypeVar
 
+import cattrs.strategies
 from attr import AttrsInstance
 from cattrs import BaseConverter
 from cattrs.strategies import (
     include_subclasses as _include_subclasses,
 )
-import cattrs.strategies
 
 from .converters import converters
 
@@ -14,7 +13,10 @@ _C = TypeVar("_C", bound=AttrsInstance)
 
 
 def include_type(tag_name: str = "class") -> Callable[[Any, BaseConverter], Any]:
-    return partial(cattrs.strategies.configure_tagged_union, tag_name=tag_name)
+    def _include_type(union: Any, converter: BaseConverter) -> Any:
+        cattrs.strategies.configure_tagged_union(converter, union, tag_name=tag_name)
+
+    return _include_type
 
 
 def include_subclasses(
